# ../addons/eventscripts/physcannon/physcannon.py


""" IMPORTS """
import es
import vecmath

import psyco
psyco.full()


""" INFORMATIONS """
informations = es.AddonInfo()
informations.name = 'PhysCannon for CSS OrangeBox'
informations.basename = 'physcannon'
informations.version = '1.0'
informations.author = 'L\'In20Cible'
informations.description = 'Allows players to grab and throw physic objects with thein default pistol.'
informations.url = 'http://addons.eventscripts.com/addons/view/physcannon/'

es.ServerVar('physcannon_version', '%s by %s' % (informations.version, informations.author), informations.description).makepublic()


""" GLOBALS """
players = dict()


""" INITIALIZATION """
def load():
    es.doblock("corelib/noisy_on")
    es.addons.registerTickListener(TickListenerCallBack)
    es.addons.registerClientCommandFilter(ClientCommandFilterCallBack)
    
def unload():
    es.doblock("corelib/noisy_off")
    es.addons.unregisterTickListener(TickListenerCallBack)
    es.addons.unregisterClientCommandFilter(ClientCommandFilterCallBack)
    
    
""" UTILITIES """
def grabViewProp(userid, index):
    if index.isdigit() and es.entitygetvalue(index, 'classname') == 'prop_physics_multiplayer':
        userid = int(userid)
        if vecmath.distance(es.getplayerlocation(userid), es.getindexprop(index, 'CBaseEntity.m_vecOrigin')) <= 256.00000:
            players[userid] = index
            es.emitsound('player', userid, 'weapons/physcannon/physcannon_pickup.wav', 0.5, 0.5)
            
def pushPhysicProp(index, vector):
    userid = es.getuserid()
    if userid and es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') != None:
        targetname = es.entitygetvalue(index, 'targetname')
        es.entitysetvalue(index, 'targetname', index)
        es.server.insertcmd('es_xfire %s %s wake;' % (userid, index) +
            'es_xfire %s %s enablemotion;' % (userid, index) +
            'es_xphysics active setvelocity %s %s;' % (index, vector) +
            'es_xfire %s %s addoutput "targetname %s"' % (userid, index, targetname))
            
            
""" CALLBACKS """
def ClientCommandFilterCallBack(userid, arguments):
    returnvalue = True
    if arguments[0].lower() == 'drop' and userid in players:
        del players[userid]
        es.emitsound('player', userid, 'weapons/physcannon/physcannon_drop.wav', 0.5, 0.5)
        returnvalue = False
    return returnvalue
    
def TickListenerCallBack():
    for userid in es.getUseridList():
        weapon = es.createplayerlist(userid)[userid]['weapon']
        if weapon and weapon[7:] in ('usp', 'glock'):
            if userid in players:
                index = players[userid]
                if es.exists('userid', userid) and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag') and es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') != None:
                    origin = tuple(es.getplayerprop(userid, 'CBasePlayer.localdata.m_vecViewOffset[%s]' % x) + y for x, y in enumerate(es.getplayerlocation(userid)))
                    direction = vecmath.viewvector(vecmath.vector(es.getplayerprop(userid, 'CCSPlayer.m_angEyeAngles[0]'), es.getplayerprop(userid, 'CCSPlayer.m_angEyeAngles[1]')))
                    distance = vecmath.distance(es.getindexprop(index, 'CPhysicsPropMultiplayer.m_collisionMins'), es.getindexprop(index, 'CPhysicsPropMultiplayer.m_collisionMaxs')) * 1.5
                    if distance < 64.00000:
                        distance = 64.00000
                    movement = es.getplayermovement(userid)[0]
                    if movement > 0.00000:
                        distance = distance + 64.00000
                    elif movement < 0.00000:
                        distance = distance - 32.00000
                    velocity = vecmath.vector(es.createvectorfrompoints(es.getindexprop(index, 'CBaseEntity.m_vecOrigin'), es.createvectorstring(origin[0] + direction[0] * distance, origin[1] + direction[1] * distance, origin[2] + direction[2] * distance)))
                    pushPhysicProp(index, velocity.setlength(velocity.length() * (10 * es.getplayerprop(userid, 'CBasePlayer.localdata.m_flLaggedMovementValue'))))
                else:
                    del players[userid]
            es.setplayerprop(userid, 'CCSPlayer.baseclass.baseclass.bcc_localdata.m_flNextAttack', es.getplayerprop(userid, 'CCSPlayer.baseclass.baseclass.bcc_localdata.m_flNextAttack') + 1)
        elif userid in players:
            del players[userid]
            es.emitsound('player', userid, 'weapons/physcannon/physcannon_drop.wav', 0.5, 0.5)
            
            
""" EVENTS """
def client_keypress(event_var):
    if event_var['keyname'] == 'attack' and event_var['es_userweapon'][7:] in ('usp', 'glock'):
        userid = int(event_var['userid'])
        if int(event_var['status']):
            if userid not in players:
                es.server.insertcmd('xgetviewprop temp %s;' % userid +
                    'es xcallblock physcannon/grabViewProp %s server_var(temp)' % userid)
        elif userid in players:
            es.emitsound('player', userid, 'weapons/physcannon/superphys_launch1.wav', 0.5, 0.5)
            pushPhysicProp(players[userid], vecmath.viewvector(vecmath.vector(es.getplayerprop(userid, 'CCSPlayer.m_angEyeAngles[0]'), es.getplayerprop(userid, 'CCSPlayer.m_angEyeAngles[1]'))) * 2048)
            del players[userid]
            
def es_map_start(event_var):
    players.clear()
    
def round_start(event_var):
    players.clear()